/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global console, tizen, window, document, Event */

/**
 * Model module.
 *
 * @module app.model
 * @namespace app.model
 */

// make sure that "app" namespace is created
window.app = window.app || {};
window.app.model = window.app.model || {};

(function model(app) {

    'use strict';

    /**
     * MIME types available for view operation.
     *
     * @memberof app.model
     * @private
     * @const {object}
     */
    var MIME_TYPES = {
            image: 'image',
            video: 'video',
            audio: 'audio',
            text: 'text'
        },

        /**
         * MIME type describing any content type.
         *
         * @memberof app.model
         * @private
         * @const {string}
         */
        ANY_MIME_TYPE = '*/*',

        /**
         * Email subject.
         *
         * @memberof app.model
         * @private
         * @const {string}
         */
        EMAIL_SUBJECT = 'Test subject',

        /**
         * Current operation name.
         *
         * @memberof app.model
         * @private
         * @type {string}
         */
        operation = null,

        /**
         * Current MIME type.
         *
         * @memberof app.model
         * @private
         * @type {string}
         * @default
         */
        currentMimeType = null,

        /**
         * Applications found compatible with operation.
         *
         * @memberof app.model
         * @private
         * @type {ApplicationControl[]}
         */
        applications = [],

        /**
         * Stores path to found image.
         *
         * @memberof app.model
         * @private
         * @type {string}
         */
        itemURI = '';

    /**
     * Handles find content success callback.
     * Stores first image's URI or fires event if there is no one.
     *
     * @memberof app.model
     * @private
     * @param {Content[]} items
     * @fires "model.no.images"
     */
    function onFindContentSuccess(items) {
        if (items.length) {
            itemURI = items[0].contentURI;
        } else {
            itemURI = '';
            document.dispatchEvent(
                new Event('model.no.images')
            );
        }
        findApplications(onApplicationFound);
    }

    /**
     * Finds images.
     *
     * @memberof app.model
     * @private
     */
    function findContent() {
        tizen.content.find(
            onFindContentSuccess,
            onError,
            null,
            new tizen.AttributeFilter(
                'mimeType',
                'CONTAINS',
                currentMimeType + '/'
            ),
            new tizen.SortMode('title', 'ASC')
        );
    }

    /**
     * Handles model errors.
     * Writes errors to console.
     *
     * @memberof app.model
     * @private
     * @param {Error} err
     */
    function onError(err) {
        console.error(err);

        if (err.name === 'NotFoundError') {
            applications = [];

            document.dispatchEvent(
                new Event('model.no.matching.applications.found')
            );
        }
    }

    /**
     * Returns applications list.
     *
     * @memberof app.model
     * @public
     * @returns {ApplicationControl[]}
     */
    function getApplications() {
        return applications;
    }

    /**
     * Handles finding applications success callback.
     *
     * @memberof app.model
     * @private
     * @param {ApplicationInformation[]} appInfo Found applications.
     * @fires "model.applications.list.updated"
     */
    function onApplicationFound(appInfo) {
        applications = appInfo;

        document.dispatchEvent(
            new Event('model.applications.list.updated')
        );
    }

    /**
     * Gets applications matching provided operation.
     *
     * @memberof app.model
     * @private
     * @param {function} callback
     */
    function findApplications(callback) {
        var appControl = null;

        try {
            appControl = new tizen.ApplicationControl(
                'http://tizen.org/appcontrol/operation/' +
                    operation,
                null,
                currentMimeType,
                null,
                null
            );
        } catch (e) {
            onError(e);
        }

        tizen.application.findAppControl(appControl, callback, onError);
    }

    /**
     * Updates operation and finds application matching this operation.
     *
     * @memberof app.model
     * @public
     * @param {object} data
     */
    function changeOperation(data) {
        operation = data.operation;
        currentMimeType = MIME_TYPES[data.mime] || null;

        if (operation === 'view') {
            findContent();
        } else {
            findApplications(onApplicationFound);
        }
    }

    /**
     * Starts application.
     *
     * @memberof app.model
     * @public
     * @param {number} applicationIndex
     */
    function launchApplication(applicationIndex) {
        var appControl = new tizen.ApplicationControl(
                'http://tizen.org/appcontrol/operation/' +
                    operation,
                itemURI,
                ANY_MIME_TYPE,
                null,
                null
            );

        tizen.application.launchAppControl(
            appControl,
            applications[applicationIndex].id,
            function onApplicationOperationSuccess() {},
            onError,
            null
        );
    }

    /**
     * Opens system email composer.
     *
     * @memberof app.model
     * @public
     * @param {object} data
     */
    function sendEmail(data) {
        var appControlData = [
            new tizen.ApplicationControlData(
                'http://tizen.org/appcontrol/data/subject',
                [EMAIL_SUBJECT]
            ),
            new tizen.ApplicationControlData(
                'http://tizen.org/appcontrol/data/text',
                [data.body]
            )
        ];

        tizen.application.launchAppControl(
            new tizen.ApplicationControl(
                'http://tizen.org/appcontrol/operation/compose',
                'mailto:' + data.to,
                null,
                null,
                appControlData,
                'GROUP'
            ),
            null,
            function onSuccess() { },
            onError,
            null
        );
    }

    /**
     * Returns defined MIME types.
     *
     * @memberof app.model
     * @public
     * @returns {object}
     */
    function getMimeTypes() {
        return MIME_TYPES;
    }

    app.model = {
        getApplications: getApplications,
        getMimeTypes: getMimeTypes,
        changeOperation: changeOperation,
        launchApplication: launchApplication,
        sendEmail: sendEmail
    };

}(window.app));
